//
//  InstanceTimelineViewController.swift
//  Tusker
//
//  Created by Shadowfacts on 12/19/19.
//  Copyright © 2019 Shadowfacts. All rights reserved.
//

import UIKit

protocol InstanceTimelineViewControllerDelegate: class {
    func didSaveInstance(url: URL)
    func didUnsaveInstance(url: URL)
}

class InstanceTimelineViewController: TimelineTableViewController {
    
    weak var delegate: InstanceTimelineViewControllerDelegate?

    weak var parentMastodonController: MastodonController?
    
    let instanceURL: URL
    let instanceMastodonController: MastodonController
    
    var toggleSaveButton: UIBarButtonItem!
    var toggleSaveButtonTitle: String {
        if SavedDataManager.shared.isSaved(instance: instanceURL, for: parentMastodonController!.accountInfo!) {
            return NSLocalizedString("Unsave", comment: "unsave instance button")
        } else {
            return NSLocalizedString("Save", comment: "save instance button")
        }
    }
    
    init(for url: URL, parentMastodonController: MastodonController) {
        self.parentMastodonController = parentMastodonController
        
        self.instanceURL = url

        // the timeline VC only stores a weak reference to it, so we need to store a strong reference to make sure it's not released immediately
        instanceMastodonController = MastodonController(instanceURL: url)

        super.init(for: .instance(instanceURL: url), mastodonController: instanceMastodonController)
    }
    
    required init?(coder aDecoder: NSCoder) {
        fatalError("init(coder:) has not been implemented")
    }
    
    override func viewDidLoad() {
        super.viewDidLoad()
        
        toggleSaveButton = UIBarButtonItem(title: toggleSaveButtonTitle, style: .plain, target: self, action: #selector(toggleSaveButtonPressed))
        navigationItem.rightBarButtonItem = toggleSaveButton
        
        NotificationCenter.default.addObserver(self, selector: #selector(savedInstancesChanged), name: .savedInstancesChanged, object: nil)
    }
    
    @objc func savedInstancesChanged() {
        toggleSaveButton.title = toggleSaveButtonTitle
    }
    
    // MARK: - Table view data source
    
    override func tableView(_ tableView: UITableView, cellForRowAt indexPath: IndexPath) -> UITableViewCell {
        let cell = super.tableView(tableView, cellForRowAt: indexPath) as! TimelineStatusTableViewCell
        cell.delegate = nil
        cell.overrideMastodonController = mastodonController
        return cell
    }
    
    // MARK: - Table view delegate
    
    override func tableView(_ tableView: UITableView, didSelectRowAt indexPath: IndexPath) {
        // no-op, we don't currently support viewing whole conversations from other instances
    }
    
    // MARK: - Interaction
    @objc func toggleSaveButtonPressed() {
        if SavedDataManager.shared.isSaved(instance: instanceURL, for: parentMastodonController!.accountInfo!) {
            SavedDataManager.shared.remove(instance: instanceURL, for: parentMastodonController!.accountInfo!)
            delegate?.didUnsaveInstance(url: instanceURL)
        } else {
            SavedDataManager.shared.add(instance: instanceURL, for: parentMastodonController!.accountInfo!)
            delegate?.didSaveInstance(url: instanceURL)
        }
    }

}
